import { ID } from '@rokoli/bnb/drf'
import { sort } from 'fast-sort'
import { computed, MaybeRefOrGetter, Ref, ref, toRef, toValue } from 'vue'

function _sort<T extends { isActive?: boolean }>(items: T[], keys: (keyof T)[]): T[] {
  return sort(items).by([
    { asc: (item) => ('isActive' in item ? (item.isActive ? 0 : 1) : 0) },
    ...keys.map((key) => ({ asc: (item: T) => item[key] })),
  ])
}

export function useItems<T extends { id: ID; isActive?: boolean }, K = T['id']>(
  objects: MaybeRefOrGetter<T[] | Map<K, T>>,
  usedIds: MaybeRefOrGetter<undefined | null | K | K[]>,
  options?: {
    save?: (id: K[]) => Promise<void>
    sortBy?: MaybeRefOrGetter<keyof T | (keyof T)[]>
  },
) {
  const sortByKeys = toRef(options?.sortBy ?? undefined) as Ref<keyof T | (keyof T)[] | undefined>
  const sortByKeysList = computed<(keyof T)[]>(() => {
    const _value = toValue(sortByKeys)
    if (_value === undefined) {
      return []
    } else if (Array.isArray(_value)) {
      return Array.from(_value)
    } else {
      return [_value]
    }
  })

  const idMap = computed<Map<K, T>>(() => {
    const value = toValue(objects)
    return Array.isArray(value) ? new Map(value.map((obj) => [obj.id as K, obj])) : value
  })
  const choices = computed<T[]>(() => {
    const value = toValue(objects)
    const choices = Array.isArray(value) ? value : Array.from(value.values())
    return _sort(
      choices.filter((item) => item?.isActive !== false),
      sortByKeysList.value,
    )
  })
  const isSaving = ref(false)
  const usedItems = computed({
    get() {
      const value = toValue(usedIds)
      return _sort(
        (Array.isArray(value) ? value : value ? [value] : [])
          .filter((id) => idMap.value.has(id))
          .map((id) => idMap.value.get(id) as T),
        sortByKeysList.value,
      )
    },
    set(value: T[]) {
      const save = options?.save
      if (save) {
        isSaving.value = true
        save(value.map((item) => item.id as K)).finally(() => (isSaving.value = false))
      }
    },
  })
  const usedItem = computed(() => usedItems.value[0])
  return { choices, usedItem, usedItems, isSaving }
}
