import { APIListPaginated, APIRetrieve, createExtendableAPI } from '@rokoli/bnb/drf'
import { defineStore } from 'pinia'

import { createTankURL } from '@/api'
import { tankAuthInit } from '@/stores/auth'
import { Playlist } from '@/types'

/**
 * Calculates the duration of a playlist.
 * May return null if the playlist contains entries with an invalid duration.
 * @param playlist
 */
export function calculatePlaylistDurationInSeconds(playlist: Playlist) {
  let duration = 0
  for (const entry of playlist.entries) {
    // entry.duration may be null/NaN if the entry references
    // a stream or other resources without an inherent duration
    if (typeof entry.duration !== 'number' || isNaN(entry.duration)) {
      return null
    }
    // entry.duration is in nanoseconds
    duration += entry.duration / 1_000_000_000
  }

  return duration
}

export const usePlaylistStore = defineStore('playlists', () => {
  const endpoint = createTankURL.prefix('playlists')
  const { api, base } = createExtendableAPI<Playlist>(endpoint, tankAuthInit)
  const { list } = APIListPaginated(api)
  return {
    ...base,
    list,
    ...APIRetrieve(api),
  }
})
